@@ -9,14 +9,14 @@
  * separate folders.
  */
 
-#include <pthread.h>
-
-/**
- * \brief cache data type
- */
 typedef struct Cache Cache;
 
 #include "link.h"
+#include "network.h"
+
+#include <stdio.h>
+#include <stdint.h>
+#include <pthread.h>
 
 /**
  * \brief Type definition for a cache segment
@@ -27,6 +27,9 @@ typedef uint8_t Seg;
  * \brief cache data type in-memory data structure
  */
 struct Cache {
+    /** \brief How many times the cache has been opened */
+    int cache_opened;
+
     /** \brief the FILE pointer for the data file*/
     FILE *dfp;
     /** \brief the FILE pointer for the metadata */
@@ -110,7 +113,7 @@ Cache *Cache_open(const char *fn);
  * \brief Close a cache data structure
  * \note This function is called by fs_release()
  */
-void Cache_close(Cache * cf);
+void Cache_close(Cache *cf);
 
 /**
  * \brief create a cache file set if it doesn't exist already
@@ -138,6 +141,6 @@ void Cache_delete(const char *fn);
  * \return the length of the segment the cache system managed to obtain.
  * \note Called by fs_read(), verified to be working
  */
-long Cache_read(Cache * cf, char *const output_buf, const off_t len,
+long Cache_read(Cache *cf, char *const output_buf, const off_t len,
                 const off_t offset_start);
 #endif
