@@ -1,9 +1,8 @@
 #include "link.h"
 
-#include "cache.h"
-#include "network.h"
-#include "sonic.h"
 #include "log.h"
+#include "memcache.h"
+#include "util.h"
 
 #include <gumbo.h>
 
@@ -42,8 +41,7 @@ static Link *Link_new(const char *linkname, LinkType type)
     /*
      * remove the '/' from linkname if it exists
      */
-    char *c =
-        &(link->linkname[strnlen(link->linkname, MAX_FILENAME_LEN) - 1]);
+    char *c = &(link->linkname[strnlen(link->linkname, MAX_FILENAME_LEN) - 1]);
     if (*c == '/') {
         *c = '\0';
     }
@@ -51,7 +49,7 @@ static Link *Link_new(const char *linkname, LinkType type)
     return link;
 }
 
-static CURL *Link_to_curl(Link * link)
+static CURL *Link_to_curl(Link *link)
 {
     CURL *curl = curl_easy_init();
     if (!curl) {
@@ -60,64 +58,156 @@ static CURL *Link_to_curl(Link * link)
     /*
      * set up some basic curl stuff
      */
-    curl_easy_setopt(curl, CURLOPT_USERAGENT, CONFIG.user_agent);
-    curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+    CURLcode ret =
+        curl_easy_setopt(curl, CURLOPT_USERAGENT, CONFIG.user_agent);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
     /*
      * for following directories without the '/'
      */
-    curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 2);
-    curl_easy_setopt(curl, CURLOPT_URL, link->f_url);
-    curl_easy_setopt(curl, CURLOPT_TCP_KEEPALIVE, 1);
-    curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
-    curl_easy_setopt(curl, CURLOPT_SHARE, CURL_SHARE);
-    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_memory_callback);
+    ret = curl_easy_setopt(curl, CURLOPT_MAXREDIRS, 2);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_URL, link->f_url);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_TCP_KEEPALIVE, 1);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 15);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_SHARE, CURL_SHARE);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret =
+        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_memory_callback);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    if (CONFIG.cafile) {
+        /*
+         * Having been given a certificate file, disable any search directory
+         * built into libcurl, so that we exclusively use the explicitly given
+         * certificate(s).
+         *
+         * If we ever add a CAPATH option, we should do the mirror for CAINFO,
+         * too: disable both and then enable whichever one(s) were given.
+         */
+        ret = curl_easy_setopt(curl, CURLOPT_CAPATH, NULL);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+
+        ret = curl_easy_setopt(curl, CURLOPT_CAINFO, CONFIG.cafile);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+    }
     if (CONFIG.insecure_tls) {
-        curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
+        ret = curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+    }
+
+    if (CONFIG.log_type & libcurl_debug) {
+        ret = curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
     }
-    // curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
 
     if (CONFIG.http_username) {
-        curl_easy_setopt(curl, CURLOPT_USERNAME, CONFIG.http_username);
+        ret = curl_easy_setopt(curl, CURLOPT_USERNAME, CONFIG.http_username);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
     }
 
     if (CONFIG.http_password) {
-        curl_easy_setopt(curl, CURLOPT_PASSWORD, CONFIG.http_password);
+        ret = curl_easy_setopt(curl, CURLOPT_PASSWORD, CONFIG.http_password);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
     }
 
     if (CONFIG.proxy) {
-        curl_easy_setopt(curl, CURLOPT_PROXY, CONFIG.proxy);
+        ret = curl_easy_setopt(curl, CURLOPT_PROXY, CONFIG.proxy);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
     }
 
     if (CONFIG.proxy_username) {
-        curl_easy_setopt(curl, CURLOPT_PROXYUSERNAME,
-                         CONFIG.proxy_username);
+        ret = curl_easy_setopt(curl, CURLOPT_PROXYUSERNAME,
+                               CONFIG.proxy_username);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
     }
 
     if (CONFIG.proxy_password) {
-        curl_easy_setopt(curl, CURLOPT_PROXYPASSWORD,
-                         CONFIG.proxy_password);
+        ret = curl_easy_setopt(curl, CURLOPT_PROXYPASSWORD,
+                               CONFIG.proxy_password);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+    }
+
+    if (CONFIG.proxy_cafile) {
+        /* See CONFIG.cafile above */
+        ret = curl_easy_setopt(curl, CURLOPT_PROXY_CAPATH, NULL);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+
+        ret = curl_easy_setopt(curl, CURLOPT_PROXY_CAINFO,
+                               CONFIG.proxy_cafile);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
     }
 
     return curl;
 }
 
-static void Link_req_file_stat(Link * this_link)
+static void Link_req_file_stat(Link *this_link)
 {
     CURL *curl = Link_to_curl(this_link);
-    curl_easy_setopt(curl, CURLOPT_NOBODY, 1);
-    curl_easy_setopt(curl, CURLOPT_FILETIME, 1L);
+    CURLcode ret = curl_easy_setopt(curl, CURLOPT_NOBODY, 1);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_FILETIME, 1L);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
 
     /*
      * We need to put the variable on the heap, because otherwise the
      * variable gets popped from the stack as the function returns.
      *
-     * It gets freed in curl_multi_perform_once();
+     * It gets freed in curl_process_msgs();
      */
     TransferStruct *transfer = CALLOC(1, sizeof(TransferStruct));
 
     transfer->link = this_link;
     transfer->type = FILESTAT;
-    curl_easy_setopt(curl, CURLOPT_PRIVATE, transfer);
+    ret = curl_easy_setopt(curl, CURLOPT_PRIVATE, transfer);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
 
     transfer_nonblocking(curl);
 }
@@ -127,11 +217,11 @@ static void Link_req_file_stat(Link * this_link)
  * \details Try and get the stats for each link in the link table. This will get
  * repeated until the uninitialised entry count drop to zero.
  */
-static void LinkTable_uninitialised_fill(LinkTable * linktbl)
+static void LinkTable_uninitialised_fill(LinkTable *linktbl)
 {
     int u;
     char s[STATUS_LEN];
-    lprintf(debug, "LinkTable_uninitialised_fill(): ... ");
+    lprintf(debug, " ... ");
     do {
         u = 0;
         for (int i = 0; i < linktbl->num; i++) {
@@ -157,8 +247,7 @@ static void LinkTable_uninitialised_fill(LinkTable * linktbl)
                 j++;
             }
         }
-    }
-    while (u);
+    } while (u);
     if (CONFIG.log_type & debug) {
         erase_string(stderr, STATUS_LEN, s);
         fprintf(stderr, "... Done!\n");
@@ -234,7 +323,7 @@ LinkTable *LinkSystem_init(const char *raw_url)
     return ROOT_LINK_TBL;
 }
 
-void LinkTable_add(LinkTable * linktbl, Link * link)
+void LinkTable_add(LinkTable *linktbl, Link *link)
 {
     linktbl->num++;
     linktbl->links =
@@ -285,10 +374,10 @@ static int linknames_equal(char *linkname, const char *linkname_new)
      * check if the link names differ by a single '/'
      */
     if (!strncmp
-        (linkname, linkname_new, strnlen(linkname, MAX_FILENAME_LEN))) {
+            (linkname, linkname_new, strnlen(linkname, MAX_FILENAME_LEN))) {
         size_t linkname_new_len = strnlen(linkname_new, MAX_FILENAME_LEN);
         if ((linkname_new_len - strnlen(linkname, MAX_FILENAME_LEN) == 1)
-            && (linkname_new[linkname_new_len - 1] == '/')) {
+                && (linkname_new[linkname_new_len - 1] == '/')) {
             return 1;
         }
     }
@@ -299,15 +388,15 @@ static int linknames_equal(char *linkname, const char *linkname_new)
  * Shamelessly copied and pasted from:
  * https://github.com/google/gumbo-parser/blob/master/examples/find_links.cc
  */
-static void HTML_to_LinkTable(GumboNode * node, LinkTable * linktbl)
+static void HTML_to_LinkTable(GumboNode *node, LinkTable *linktbl)
 {
     if (node->type != GUMBO_NODE_ELEMENT) {
         return;
     }
     GumboAttribute *href;
     if (node->v.element.tag == GUMBO_TAG_A &&
-        (href =
-         gumbo_get_attribute(&node->v.element.attributes, "href"))) {
+            (href =
+                 gumbo_get_attribute(&node->v.element.attributes, "href"))) {
         /*
          * if it is valid, copy the link onto the heap
          */
@@ -322,8 +411,8 @@ static void HTML_to_LinkTable(GumboNode * node, LinkTable * linktbl)
             comp_len--;
         }
         if (((type == LINK_DIR) || (type == LINK_UNINITIALISED_FILE)) &&
-            !linknames_equal(linktbl->links[linktbl->num - 1]->linkname,
-                             href->value)) {
+                !linknames_equal(linktbl->links[linktbl->num - 1]->linkname,
+                                 href->value)) {
             LinkTable_add(linktbl, Link_new(href->value, type));
         }
     }
@@ -337,14 +426,26 @@ static void HTML_to_LinkTable(GumboNode * node, LinkTable * linktbl)
     return;
 }
 
-void Link_set_file_stat(Link * this_link, CURL * curl)
+void Link_set_file_stat(Link *this_link, CURL *curl)
 {
     long http_resp;
-    curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_resp);
+    CURLcode ret =
+        curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_resp);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
     if (http_resp == HTTP_OK) {
         double cl = 0;
-        curl_easy_getinfo(curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &cl);
-        curl_easy_getinfo(curl, CURLINFO_FILETIME, &(this_link->time));
+        ret =
+            curl_easy_getinfo(curl, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &cl);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+        ret =
+            curl_easy_getinfo(curl, CURLINFO_FILETIME, &(this_link->time));
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
         if (cl <= 0) {
             this_link->type = LINK_INVALID;
         } else {
@@ -362,7 +463,7 @@ void Link_set_file_stat(Link * this_link, CURL * curl)
     }
 }
 
-static void LinkTable_fill(LinkTable * linktbl)
+static void LinkTable_fill(LinkTable *linktbl)
 {
     Link *head_link = linktbl->links[0];
     for (int i = 1; i < linktbl->num; i++) {
@@ -385,7 +486,7 @@ static void LinkTable_fill(LinkTable * linktbl)
 /**
  * \brief Reset invalid links in the link table
  */
-static void LinkTable_invalid_reset(LinkTable * linktbl)
+static void LinkTable_invalid_reset(LinkTable *linktbl)
 {
     int j = 0;
     for (int i = 0; i < linktbl->num; i++) {
@@ -398,7 +499,7 @@ static void LinkTable_invalid_reset(LinkTable * linktbl)
     lprintf(debug, "%d invalid links\n", j);
 }
 
-void LinkTable_free(LinkTable * linktbl)
+void LinkTable_free(LinkTable *linktbl)
 {
     for (int i = 0; i < linktbl->num; i++) {
         FREE(linktbl->links[i]);
@@ -407,7 +508,7 @@ void LinkTable_free(LinkTable * linktbl)
     FREE(linktbl);
 }
 
-void LinkTable_print(LinkTable * linktbl)
+void LinkTable_print(LinkTable *linktbl)
 {
     if (CONFIG.log_type & info) {
         int j = 0;
@@ -423,8 +524,8 @@ void LinkTable_print(LinkTable * linktbl)
                     this_link->content_length,
                     this_link->linkname, this_link->f_url);
             if ((this_link->type != LINK_FILE)
-                && (this_link->type != LINK_DIR)
-                && (this_link->type != LINK_HEAD)) {
+                    && (this_link->type != LINK_DIR)
+                    && (this_link->type != LINK_HEAD)) {
                 j++;
             }
         }
@@ -434,44 +535,6 @@ void LinkTable_print(LinkTable * linktbl)
     }
 }
 
-DataStruct Link_to_DataStruct(Link * head_link)
-{
-    char *url = head_link->f_url;
-    CURL *curl = Link_to_curl(head_link);
-
-    DataStruct buf;
-    buf.size = 0;
-    buf.data = NULL;
-
-    curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *) &buf);
-
-    /*
-     * If we get temporary HTTP failure, wait for 5 seconds before retry
-     */
-    long http_resp = 0;
-    do {
-        transfer_blocking(curl);
-        curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_resp);
-        if (HTTP_temp_failure(http_resp)) {
-            lprintf(warning,
-                    "URL: %s, HTTP %ld, retrying later.\n",
-                    url, http_resp);
-            sleep(CONFIG.http_wait_sec);
-        } else if (http_resp != HTTP_OK) {
-            lprintf(warning,
-                    "cannot retrieve URL: %s, HTTP %ld\n", url, http_resp);
-            buf.size = 0;
-            curl_easy_cleanup(curl);
-            return buf;
-        }
-    }
-    while (HTTP_temp_failure(http_resp));
-
-    curl_easy_getinfo(curl, CURLINFO_FILETIME, &(head_link->time));
-    curl_easy_cleanup(curl);
-    return buf;
-}
-
 LinkTable *LinkTable_alloc(const char *url)
 {
     LinkTable *linktbl = CALLOC(1, sizeof(LinkTable));
@@ -493,8 +556,8 @@ LinkTable *LinkTable_new(const char *url)
     /*
      * start downloading the base URL
      */
-    DataStruct buf = Link_to_DataStruct(linktbl->links[0]);
-    if (buf.size == 0) {
+    TransferStruct ts = Link_download_full(linktbl->links[0]);
+    if (ts.curr_size == 0) {
         LinkTable_free(linktbl);
         return NULL;
     }
@@ -502,10 +565,10 @@ LinkTable *LinkTable_new(const char *url)
     /*
      * Otherwise parsed the received data
      */
-    GumboOutput *output = gumbo_parse(buf.data);
+    GumboOutput *output = gumbo_parse(ts.data);
     HTML_to_LinkTable(output->root, linktbl);
     gumbo_destroy_output(&kGumboDefaultOptions, output);
-    FREE(buf.data);
+    FREE(ts.data);
 
     int skip_fill = 0;
     char *unescaped_path;
@@ -575,7 +638,7 @@ static void LinkTable_disk_delete(const char *dirn)
     FREE(metadirn);
 }
 
-int LinkTable_disk_save(LinkTable * linktbl, const char *dirn)
+int LinkTable_disk_save(LinkTable *linktbl, const char *dirn)
 {
     char *metadirn = path_append(META_DIR, dirn);
     char *path;
@@ -677,11 +740,11 @@ LinkTable *path_to_Link_LinkTable_new(const char *path)
             next_table = LinkTable_new(link->f_url);
         } else if (CONFIG.mode == SONIC) {
             if (!CONFIG.sonic_id3) {
-                next_table = sonic_LinkTable_new_index(link->sonic_id);
+                next_table = sonic_LinkTable_new_index(link->sonic.id);
             } else {
                 next_table =
-                    sonic_LinkTable_new_id3(link->sonic_depth,
-                                            link->sonic_id);
+                    sonic_LinkTable_new_id3(link->sonic.depth,
+                                            link->sonic.id);
             }
         } else {
             lprintf(fatal, "Invalid CONFIG.mode\n");
@@ -691,7 +754,7 @@ LinkTable *path_to_Link_LinkTable_new(const char *path)
     return next_table;
 }
 
-static Link *path_to_Link_recursive(char *path, LinkTable * linktbl)
+static Link *path_to_Link_recursive(char *path, LinkTable *linktbl)
 {
     /*
      * skip the leading '/' if it exists
@@ -715,7 +778,7 @@ static Link *path_to_Link_recursive(char *path, LinkTable * linktbl)
          */
         for (int i = 1; i < linktbl->num; i++) {
             if (!strncmp
-                (path, linktbl->links[i]->linkname, MAX_FILENAME_LEN)) {
+                    (path, linktbl->links[i]->linkname, MAX_FILENAME_LEN)) {
                 /*
                  * We found our link
                  */
@@ -739,7 +802,7 @@ static Link *path_to_Link_recursive(char *path, LinkTable * linktbl)
         char *next_path = slash + 1;
         for (int i = 1; i < linktbl->num; i++) {
             if (!strncmp
-                (path, linktbl->links[i]->linkname, MAX_FILENAME_LEN)) {
+                    (path, linktbl->links[i]->linkname, MAX_FILENAME_LEN)) {
                 /*
                  * The next sub-directory exists
                  */
@@ -752,13 +815,13 @@ static Link *path_to_Link_recursive(char *path, LinkTable * linktbl)
                         if (!CONFIG.sonic_id3) {
                             next_table =
                                 sonic_LinkTable_new_index
-                                (linktbl->links[i]->sonic_id);
+                                (linktbl->links[i]->sonic.id);
                         } else {
                             next_table =
                                 sonic_LinkTable_new_id3
                                 (linktbl->links
-                                 [i]->sonic_depth,
-                                 linktbl->links[i]->sonic_id);
+                                 [i]->sonic.depth,
+                                 linktbl->links[i]->sonic.id);
                         }
                     } else {
                         lprintf(fatal, "Invalid CONFIG.mode\n");
@@ -791,75 +854,179 @@ Link *path_to_Link(const char *path)
     return link;
 }
 
-long
-path_download(const char *path, char *output_buf, size_t size,
-              off_t offset)
+TransferStruct Link_download_full(Link *link)
 {
-    if (!path) {
-        lprintf(fatal, "\npath_download(): NULL path supplied\n");
+    char *url = link->f_url;
+    CURL *curl = Link_to_curl(link);
+
+    TransferStruct ts;
+    ts.curr_size = 0;
+    ts.data = NULL;
+    ts.type = DATA;
+    ts.transferring = 1;
+
+    CURLcode ret = curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *) &ts);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
     }
-    Link *link;
-    link = path_to_Link(path);
+    ret = curl_easy_setopt(curl, CURLOPT_PRIVATE, (void *) &ts);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+
+    /*
+     * If we get temporary HTTP failure, wait for 5 seconds before retry
+     */
+    long http_resp = 0;
+    do {
+        transfer_blocking(curl);
+        ret = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_resp);
+        if (ret) {
+            lprintf(error, "%s", curl_easy_strerror(ret));
+        }
+        if (HTTP_temp_failure(http_resp)) {
+            lprintf(warning,
+                    "URL: %s, HTTP %ld, retrying later.\n",
+                    url, http_resp);
+            sleep(CONFIG.http_wait_sec);
+        } else if (http_resp != HTTP_OK) {
+            lprintf(warning,
+                    "cannot retrieve URL: %s, HTTP %ld\n", url, http_resp);
+            ts.curr_size = 0;
+            free(ts.data); /* not FREE(); can be NULL on error path! */
+            curl_easy_cleanup(curl);
+            return ts;
+        }
+    } while (HTTP_temp_failure(http_resp));
+
+    ret = curl_easy_getinfo(curl, CURLINFO_FILETIME, &(link->time));
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    curl_easy_cleanup(curl);
+    return ts;
+}
+
+static CURL *Link_download_curl_setup(Link *link, size_t req_size, off_t offset,
+                                      TransferStruct *header,
+                                      TransferStruct *ts)
+{
     if (!link) {
-        return -ENOENT;
+        lprintf(fatal, "Invalid supplied\n");
     }
 
     size_t start = offset;
-    size_t end = start + size;
+    size_t end = start + req_size;
+
     char range_str[64];
     snprintf(range_str, sizeof(range_str), "%lu-%lu", start, end);
-    lprintf(debug, "path_download(%s, %s);\n", path, range_str);
-
-    DataStruct buf;
-    buf.size = 0;
-    buf.data = NULL;
+    lprintf(debug, "%s: %s\n", link->linkname, range_str);
 
     CURL *curl = Link_to_curl(link);
-    curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *) &buf);
-    curl_easy_setopt(curl, CURLOPT_RANGE, range_str);
+    CURLcode ret =
+        curl_easy_setopt(curl, CURLOPT_HEADERDATA, (void *) header);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *) ts);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_PRIVATE, (void *) ts);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
+    ret = curl_easy_setopt(curl, CURLOPT_RANGE, range_str);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
 
-    DataStruct header;
-    header.size = 0;
-    header.data = NULL;
-    curl_easy_setopt(curl, CURLOPT_HEADERDATA, (void *) &header);
-
-    transfer_blocking(curl);
+    return curl;
+}
 
+static curl_off_t Link_download_cleanup(CURL *curl, TransferStruct *header)
+{
     /*
      * Check for range seek support
      */
     if (!CONFIG.no_range_check) {
-        if (!strcasestr((header.data), "Accept-Ranges: bytes")) {
+        if (!strcasestr((header->data), "Accept-Ranges: bytes")) {
             fprintf(stderr, "This web server does not support HTTP \
 range requests\n");
             exit(EXIT_FAILURE);
         }
     }
 
-    FREE(header.data);
+    FREE(header->data);
 
     long http_resp;
-    curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_resp);
+    CURLcode ret = curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_resp);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
+    }
     if (!((http_resp != HTTP_OK) ||
-          (http_resp != HTTP_PARTIAL_CONTENT) ||
-          (http_resp != HTTP_RANGE_NOT_SATISFIABLE))) {
-        lprintf(warning,
-                "Could not download %s, HTTP %ld\n",
-                link->f_url, http_resp);
+            (http_resp != HTTP_PARTIAL_CONTENT) ||
+            (http_resp != HTTP_RANGE_NOT_SATISFIABLE))) {
+        char *url;
+        curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &url);
+        lprintf(warning, "Could not download %s, HTTP %ld\n", url, http_resp);
         return -ENOENT;
     }
 
-    double dl;
-    curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD, &dl);
-
-    size_t recv = dl;
-    if (recv > size) {
-        recv = size;
+    curl_off_t recv;
+    ret = curl_easy_getinfo(curl, CURLINFO_SIZE_DOWNLOAD_T, &recv);
+    if (ret) {
+        lprintf(error, "%s", curl_easy_strerror(ret));
     }
 
-    memmove(output_buf, buf.data, recv);
     curl_easy_cleanup(curl);
-    FREE(buf.data);
 
     return recv;
 }
+
+long Link_download(Link *link, char *output_buf, size_t req_size, off_t offset)
+{
+    TransferStruct ts;
+    ts.curr_size = 0;
+    ts.data = NULL;
+    ts.type = DATA;
+    ts.transferring = 1;
+
+    TransferStruct header;
+    header.curr_size = 0;
+    header.data = NULL;
+
+    CURL *curl = Link_download_curl_setup(link, req_size, offset, &header, &ts);
+
+    transfer_blocking(curl);
+
+    curl_off_t recv = Link_download_cleanup(curl, &header);
+
+    /* The extra 1 byte is probably for '\0' */
+    if (recv - 1 == (long int) req_size) {
+        recv--;
+    } else if (offset + req_size < link->content_length) {
+        lprintf(error, "req_size: %lu, recv: %ld\n", req_size, recv);
+    }
+
+    memmove(output_buf, ts.data, recv);
+    FREE(ts.data);
+
+    return recv;
+}
+
+long path_download(const char *path, char *output_buf, size_t req_size,
+                   off_t offset)
+{
+    if (!path) {
+        lprintf(fatal, "NULL path supplied\n");
+    }
+
+    Link *link;
+    link = path_to_Link(path);
+    if (!link) {
+        return -ENOENT;
+    }
+
+    return Link_download(link, output_buf, req_size, offset);
+}
