@@ -6,8 +6,12 @@
  * \brief network related functions
  */
 
+typedef struct TransferStruct TransferStruct;
+
 #include "link.h"
 
+#include <curl/curl.h>
+
 /** \brief HTTP response codes */
 typedef enum {
     HTTP_OK = 200,
@@ -28,15 +32,10 @@ int curl_multi_perform_once(void);
 void NetworkSystem_init(void);
 
 /** \brief blocking file transfer */
-void transfer_blocking(CURL * curl);
+void transfer_blocking(CURL *curl);
 
 /** \brief non blocking file transfer */
-void transfer_nonblocking(CURL * curl);
-
-/** \brief callback function for file transfer */
-size_t
-write_memory_callback(void *contents, size_t size, size_t nmemb,
-                      void *userp);
+void transfer_nonblocking(CURL *curl);
 
 /**
  * \brief check if a HTTP response code corresponds to a temporary failure
