@@ -1,15 +1,17 @@
-#include "config.h"
 #include "util.h"
+
+#include "config.h"
 #include "log.h"
 
 #include <openssl/md5.h>
 #include <uuid/uuid.h>
 
+#include <errno.h>
 #include <execinfo.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
+#include <unistd.h>
+
 
 /**
  * \brief Backtrace buffer size
@@ -31,7 +33,7 @@ char *path_append(const char *path, const char *filename)
 {
     int needs_separator = 0;
     if ((path[strnlen(path, MAX_PATH_LEN) - 1] != '/')
-        && (filename[0] != '/')) {
+            && (filename[0] != '/')) {
         needs_separator = 1;
     }
 
@@ -52,25 +54,23 @@ int64_t round_div(int64_t a, int64_t b)
     return (a + (b / 2)) / b;
 }
 
-void PTHREAD_MUTEX_UNLOCK(pthread_mutex_t * x)
+void PTHREAD_MUTEX_UNLOCK(pthread_mutex_t *x)
 {
     int i;
     i = pthread_mutex_unlock(x);
     if (i) {
         lprintf(fatal,
-                "thread %x: pthread_mutex_unlock() failed, %d, %s\n",
-                pthread_self(), i, strerror(i));
+                "thread %x: %d, %s\n", pthread_self(), i, strerror(i));
     }
 }
 
-void PTHREAD_MUTEX_LOCK(pthread_mutex_t * x)
+void PTHREAD_MUTEX_LOCK(pthread_mutex_t *x)
 {
     int i;
     i = pthread_mutex_lock(x);
     if (i) {
         lprintf(fatal,
-                "thread %x: pthread_mutex_lock() failed, %d, %s\n",
-                pthread_self(), i, strerror(i));
+                "thread %x: %d, %s\n", pthread_self(), i, strerror(i));
     }
 }
 
@@ -88,7 +88,7 @@ void exit_failure(void)
     exit(EXIT_FAILURE);
 }
 
-void erase_string(FILE * file, size_t max_len, char *s)
+void erase_string(FILE *file, size_t max_len, char *s)
 {
     size_t l = strnlen(s, max_len);
     for (size_t k = 0; k < l; k++) {
@@ -142,9 +142,8 @@ void FREE(void *ptr)
 {
     if (ptr) {
         free(ptr);
-        ptr = NULL;
     } else {
-        lprintf(fatal, "attempted to double free a pointer!\n");
+        lprintf(fatal, "attempted to free NULL ptr!\n");
     }
 }
 
